@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's initially known functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <vector>
@@ -339,7 +339,7 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			ex ser;
 			// manually construct the primitive expansion
 			for (int i=1; i<order; ++i)
-				ser += pow(s,i) / pow(numeric(i), _num2);
+				ser += pow(s,i) / pow(numeric(i), *_num2_p);
 			// substitute the argument's series expansion
 			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
 			// maybe that was terminating, so add a proper order term
@@ -490,14 +490,14 @@ static ex binomial_sym(const ex & x, const numeric & y)
 	if (y.is_integer()) {
 		if (y.is_nonneg_integer()) {
 			const unsigned N = y.to_int();
-			if (N == 0) return _num0;
+			if (N == 0) return _ex0;
 			if (N == 1) return x;
 			ex t = x.expand();
 			for (unsigned i = 2; i <= N; ++i)
 				t = (t * (x + i - y - 1)).expand() / i;
 			return t;
 		} else
-			return _num0;
+			return _ex0;
 	}
 
 	return binomial(x, y).hold();
@@ -650,6 +650,106 @@ ex lsolve(const ex &eqns, const ex &symbols, unsigned options)
 	return sollist;
 }
 
+//////////
+// Find real root of f(x) numerically
+//////////
+
+const numeric
+fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
+{
+	if (!x1.is_real() || !x2.is_real()) {
+		throw std::runtime_error("fsolve(): interval not bounded by real numbers");
+	}
+	if (x1==x2) {
+		throw std::runtime_error("fsolve(): vanishing interval");
+	}
+	// xx[0] == left interval limit, xx[1] == right interval limit.
+	// fx[0] == f(xx[0]), fx[1] == f(xx[1]).
+	// We keep the root bracketed: xx[0]<xx[1] and fx[0]*fx[1]<0.
+	numeric xx[2] = { x1<x2 ? x1 : x2,
+	                  x1<x2 ? x2 : x1 };
+	ex f;
+	if (is_a<relational>(f_in)) {
+		f = f_in.lhs()-f_in.rhs();
+	} else {
+		f = f_in;
+	}
+	const ex fx_[2] = { f.subs(x==xx[0]).evalf(),
+	                    f.subs(x==xx[1]).evalf() };
+	if (!is_a<numeric>(fx_[0]) || !is_a<numeric>(fx_[1])) {
+		throw std::runtime_error("fsolve(): function does not evaluate numerically");
+	}
+	numeric fx[2] = { ex_to<numeric>(fx_[0]),
+	                  ex_to<numeric>(fx_[1]) };
+	if (!fx[0].is_real() || !fx[1].is_real()) {
+		throw std::runtime_error("fsolve(): function evaluates to complex values at interval boundaries");
+	}
+	if (fx[0]*fx[1]>=0) {
+		throw std::runtime_error("fsolve(): function does not change sign at interval boundaries");
+	}
+
+	// The Newton-Raphson method has quadratic convergence!  Simply put, it
+	// replaces x with x-f(x)/f'(x) at each step.  -f/f' is the delta:
+	const ex ff = normal(-f/f.diff(x));
+	int side = 0;  // Start at left interval limit.
+	numeric xxprev;
+	numeric fxprev;
+	do {
+		xxprev = xx[side];
+		fxprev = fx[side];
+		xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf());
+		fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf());
+		if ((side==0 && xx[0]<xxprev) || (side==1 && xx[1]>xxprev) || xx[0]>xx[1]) {
+			// Oops, Newton-Raphson method shot out of the interval.
+			// Restore, and try again with the other side instead!
+			xx[side] = xxprev;
+			fx[side] = fxprev;
+			side = !side;
+			xxprev = xx[side];
+			fxprev = fx[side];
+			xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf());
+			fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf());
+		}
+		if ((fx[side]<0 && fx[!side]<0) || (fx[side]>0 && fx[!side]>0)) {
+			// Oops, the root isn't bracketed any more.
+			// Restore, and perform a bisection!
+			xx[side] = xxprev;
+			fx[side] = fxprev;
+
+			// Ah, the bisection! Bisections converge linearly. Unfortunately,
+			// they occur pretty often when Newton-Raphson arrives at an x too
+			// close to the result on one side of the interval and
+			// f(x-f(x)/f'(x)) turns out to have the same sign as f(x) due to
+			// precision errors! Recall that this function does not have a
+			// precision goal as one of its arguments but instead relies on
+			// x converging to a fixed point. We speed up the (safe but slow)
+			// bisection method by mixing in a dash of the (unsafer but faster)
+			// secant method: Instead of splitting the interval at the
+			// arithmetic mean (bisection), we split it nearer to the root as
+			// determined by the secant between the values xx[0] and xx[1].
+			// Don't set the secant_weight to one because that could disturb
+			// the convergence in some corner cases!
+			static const double secant_weight = 0.984375;  // == 63/64 < 1
+			numeric xxmid = (1-secant_weight)*0.5*(xx[0]+xx[1])
+			    + secant_weight*(xx[0]+fx[0]*(xx[0]-xx[1])/(fx[1]-fx[0]));
+			numeric fxmid = ex_to<numeric>(f.subs(x==xxmid).evalf());
+			if (fxmid.is_zero()) {
+				// Luck strikes...
+				return xxmid;
+			}
+			if ((fxmid<0 && fx[side]>0) || (fxmid>0 && fx[side]<0)) {
+				side = !side;
+			}
+			xxprev = xx[side];
+			fxprev = fx[side];
+			xx[side] = xxmid;
+			fx[side] = fxmid;
+		}
+	} while (xxprev!=xx[side]);
+	return xxprev;
+}
+
+
 /* Force inclusion of functions from inifcns_gamma and inifcns_zeta
  * for static lib (so ginsh will see them). */
 unsigned force_include_tgamma = tgamma_SERIAL::serial;
