@@ -4,7 +4,7 @@
  *  This file must be processed with flex. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __GINAC_INPUT_LEXER_H__
@@ -37,11 +37,7 @@ extern "C" {
 extern int ginac_yyerror(char *s);
 extern int ginac_yylex(void);
 extern void ginac_yyrestart(FILE *f);
-#if YYTEXT_POINTER
 extern char *ginac_yytext;
-#else
-extern char ginac_yytext[];
-#endif
 
 namespace GiNaC {
 
