@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's overloaded operators. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <iostream>
@@ -175,7 +175,7 @@ const numeric operator+(const numeric & lh)
 
 const numeric operator-(const numeric & lh)
 {
-	return _num_1.mul(lh);
+	return _num_1_p->mul(lh);
 }
 
 
@@ -214,14 +214,14 @@ const ex operator--(ex & lh, int)
 /** Numeric prefix increment.  Adds 1 and returns incremented number. */
 numeric& operator++(numeric & rh)
 {
-	rh = rh.add(_num1);
+	rh = rh.add(*_num1_p);
 	return rh;
 }
 
 /** Numeric prefix decrement.  Subtracts 1 and returns decremented number. */
 numeric& operator--(numeric & rh)
 {
-	rh = rh.add(_num_1);
+	rh = rh.add(*_num_1_p);
 	return rh;
 }
 
@@ -230,7 +230,7 @@ numeric& operator--(numeric & rh)
 const numeric operator++(numeric & lh, int)
 {
 	numeric tmp(lh);
-	lh = lh.add(_num1);
+	lh = lh.add(*_num1_p);
 	return tmp;
 }
 
@@ -239,7 +239,7 @@ const numeric operator++(numeric & lh, int)
 const numeric operator--(numeric & lh, int)
 {
 	numeric tmp(lh);
-	lh = lh.add(_num_1);
+	lh = lh.add(*_num_1_p);
 	return tmp;
 }
 
