@@ -3,7 +3,7 @@
  *  Interface to class for extended truncated power series. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __GINAC_SERIES_H__
@@ -56,6 +56,7 @@ public:
 	ex normal(exmap & repl, exmap & rev_lookup, int level = 0) const;
 	ex expand(unsigned options = 0) const;
 	ex conjugate() const;
+	ex eval_integ() const;
 protected:
 	ex derivative(const symbol & s) const;
 
@@ -83,6 +84,10 @@ public:
 	 *  false otherwise. */
 	bool is_terminating() const;
 
+	/** Get coefficients and exponents. */
+	ex coeffop(size_t i) const;
+	ex exponop(size_t i) const;
+
 	ex add_series(const pseries &other) const;
 	ex mul_const(const numeric &other) const;
 	ex mul_series(const pseries &other) const;
